#include <avr/io.h>
#include <inttypes.h>
#include <util/delay.h>

#include "lcd.h"
#include "makra.h"

void LCD_EPULSE(void)
{
	PORT(LCD_PORT) |= 1<<LCD_E;
	_delay_us(0.5);
	PORT(LCD_PORT) &= ~(1<<LCD_E);
}
	
void LCDsendHalf(uint8_t data)
{
	data = (data & 0x0F);
	PORT(LCD_PORT) = (PORT(LCD_PORT) & ~(1<<LCD_D4 | 1<<LCD_D5 | 1<<LCD_D6 |1<<LCD_D7)) | (data);
	LCD_EPULSE();
}

void LCDsend(uint8_t data)
{
	LCDsendHalf(data>>4);
	LCDsendHalf(data);
	_delay_us(120);	
}

void LCDcommand(uint8_t command)
{
	PORT(LCD_PORT) &= ~(1<<LCD_RS);
	LCDsend(command);
}

void LCDdata(uint8_t data)
{
	PORT(LCD_PORT) |= 1<<LCD_RS;
	LCDsend(data);
}

void LCDcls(void)
{
	LCDcommand(LCDC_CLS);
	_delay_ms(4.8);	
}

void LCDhome(void)
{
	LCDcommand(LCDC_HOME);
	_delay_ms(4.8);
}

void LCDinit(void)
{
	_delay_ms(15);
	PORT(LCD_PORT) &= ~(1<<LCD_RS);
	LCDsendHalf(LCDC_FUNC|LCDC_FUNC8b);
	_delay_ms(4.1);
	LCDsendHalf(LCDC_FUNC|LCDC_FUNC8b);
	_delay_us(200);
	LCDsendHalf(LCDC_FUNC|LCDC_FUNC4b);
	_delay_us(200);
	LCDcommand(LCDC_FUNC|LCDC_FUNC4b|LCDC_FUNC2L|LCDC_FUNC5x7);
	LCDcommand(LCDC_ON);
	LCDcls();
	LCDcommand(LCDC_MODE|LCDC_MODER);
	LCDcommand(LCDC_ON|LCDC_ONDISPLAY|LCDC_ONCURSOR);
}

void LCDstr(char* str)
{
	char znak;
	while(0 != (znak = *(str++)))
	{
		LCDdata(znak);
	}
}
