#include <avr/io.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include "lcd.h"
#include "makra.h"

// Czestotliwosc taktowania DDS
#define DDS_CLK 30000000

// Wejscia syntezy DDS
#define DATA 1
#define DATA_PORT C
#define W_CLK 6
#define W_CLK_PORT D
#define FQ_UD 7
#define FQ_UD_PORT D
#define RESET 0
#define RESET_PORT B

// Wyjscia podpiete do przekaznikow
#define GAIN 5
#define SQR 4
#define GAIN_PORT C
#define SQR_PORT C

// Wyprowadzenia Impulsatora
#define IMP_PORT B
#define IMPA 6
#define IMPB 7

// Przyciski + przycisk impulsatora
#define SWSQR 3
#define SWSQR_PORT B
#define SWGAIN 1
#define SWGAIN_PORT B
#define SWIMP 2
#define SWIMP_PORT B

uint8_t dds_multiplier = 1;	//mnoznik PLL, =1 lub =6

// 5 bajtow wczytywanych do syntezy
uint8_t data4 = 0x00;	//bajt konfiguracyjny - mnoznik PLL=1, faza=0
uint8_t data3 = 0x00;	//czestotliwosc MSB
uint8_t data2 = 0x00;
uint8_t data1 = 0x00;
uint8_t data0 = 0x00;   //czestotliwosc LSB

uint32_t freq = 0;		//zadana czestotliwosc w Hz

uint8_t sw = 0x00;		//do drgan przyciskow
uint8_t adc_it = 0x00;
uint16_t adc_sum = 0;
uint16_t adc_avg = 0;
uint8_t setting = 0;	//info o wzmocnieniu i wlaczeniu TTL
uint8_t imp_stat = 0x00;	//impulsator
uint8_t position = 8;	//krok zmiany czestotliwosci, 8=1Hz, 7=10Hz itd... 1=10MHz
uint8_t curpos = 0;		//pozycja kursora na wyswietlaczu

char line1[17]="  00.000.000Hz  ";	//linie wyswietlacza
char line2[15]="      Vpp     ";

void dds_load_byte(uint8_t byte);	//ladowanie pojedynczego bajtu
void dds_load(void);	//zaladowanie 5 bajtow do dds
void dds_init(void);	//inicjacja dds - przelaczenie w szeregowy tryb
void lcd_update(void);	//wyswietla 2 linie i ustawia kursor
void dds_update(void);	//przygotowuje dane do wyslania do DDS
void freq_update(void);	//aktualizuje dane do LCD oraz wywoluje dds_update
void ampl_update(uint16_t);

int main(void)
{
	DDR(LCD_PORT) |= 1<<LCD_RS | 1<<LCD_E | 1<<LCD_D4 | 1<<LCD_D5 | 1<<LCD_D6 | 1<<LCD_D7;	// konfiguracja wyj
	
	DDR(GAIN_PORT) |= 1<<GAIN;
	DDR(SQR_PORT) |= 1<<SQR;
	PORT(GAIN_PORT) &= ~(1<<GAIN);
	PORT(SQR_PORT) &= ~(1<<SQR);
	
	DDR(SWSQR_PORT) &= ~(1<<SWSQR);
	DDR(SWGAIN_PORT) &= ~(1<<SWGAIN);
	DDR(SWIMP_PORT) &= ~(1<<SWIMP);
	PORT(SWSQR_PORT) |= 1<<SWSQR;
	PORT(SWGAIN_PORT) |= 1<<SWGAIN;
	PORT(SWIMP_PORT) |= 1<<SWIMP;
	
	DDR(IMP_PORT) &= ~(1<<IMPA | 1<<IMPB);			//impulsator
	PORT(IMP_PORT) |= (1<<IMPA | 1<<IMPB);
	
	sei();   //globalne zezwolenie na przerwania

	//timer 0 inicjalizacja
	TCCR0 = 1<<CS02; //| 1<<CS00;
	TIMSK = 1<<TOIE0;
	
	//ADC inizjalizacja
	ADMUX = 0x00;
	ADCSRA |= 1<<ADEN | 1<<ADSC | 1<<ADPS2 | 1<<ADPS1 | 1<<ADPS0;
	
	//dds - inicjalizacja
	DDR(RESET_PORT) |= 1<<RESET;
	DDR(DATA_PORT) |= 1<<DATA;
	DDR(W_CLK_PORT) |= 1<<W_CLK;
	DDR(FQ_UD_PORT) |= 1<<FQ_UD;
	PORT(RESET_PORT) &= ~(1<<RESET);
	PORT(DATA_PORT) &= ~(1<<DATA);
	PORT(W_CLK_PORT) &= ~(1<<W_CLK);
	PORT(FQ_UD_PORT) &= ~(1<<FQ_UD);

	LCDinit();						//inicjalizacja wywietlacza w 4bit
	LCDcommand(LCDC_DDA|11);
	dds_init();
	freq_update();
	lcd_update();

	while(1)
	{
		//
	}

	return 0;
}


void dds_init(void)
{
	PORT(RESET_PORT) |= 1<<RESET;
	PORT(RESET_PORT) &= ~(1<<RESET);
	PORT(W_CLK_PORT) |= 1<<W_CLK;
	PORT(W_CLK_PORT) &= ~(1<<W_CLK);
	PORT(FQ_UD_PORT) |= 1<<FQ_UD;
	PORT(FQ_UD_PORT) &= ~(1<<FQ_UD);
}

void dds_load(void)
{
	dds_load_byte(data0);
	dds_load_byte(data1);
	dds_load_byte(data2);
	dds_load_byte(data3);
	dds_load_byte(data4);
	PORT(FQ_UD_PORT) |= 1<<FQ_UD;
	PORT(FQ_UD_PORT) &= ~(1<<FQ_UD);
}

void dds_load_byte(uint8_t byte)
{
	uint8_t n;
	for(n=0;n<8;n++)
	{
		if(byte & (0x01<<n))
		{
			PORT(DATA_PORT) = PORT(DATA_PORT) | 1<<DATA;
		}
		else
		{
			PORT(DATA_PORT) = PORT(DATA_PORT) & ~(1<<DATA);
		}
		PORT(W_CLK_PORT) |= 1<<W_CLK;
		PORT(W_CLK_PORT) &= ~(1<<W_CLK);
	}
}

void sqr_update(void)
{
	if(setting & 0x01)
	{
		line2[11] = 'T';
		line2[12] = 'T';
		line2[13] = 'L';
	}
	else
	{
		line2[11] = ' ';
		line2[12] = ' ';
		line2[13] = ' ';
	}
}

void ampl_update(uint16_t adc_avg)
{
	line2[2] = adc_avg/100 + 48;
	adc_avg = adc_avg%100;
	if(setting & 0x02)
	{
		line2[3] = '.';
		line2[4] = adc_avg/10 + 48;
		adc_avg = adc_avg%10;
		line2[5] = adc_avg + 48;
	}
	else
	{
		line2[3] = adc_avg/10 + 48;
		adc_avg = adc_avg%10;
		line2[4] = adc_avg + 48;
		line2[5] = 'm';
	}
}

void lcd_update(void)
{
	LCDcommand(LCDC_DDA|0);
	LCDstr(line1);
	LCDcommand(LCDC_DDA|64);
	LCDstr(line2);
	LCDcommand(LCDC_DDA|curpos);
}

void freq_update(void)
{
	if(freq > 1000)
	{
		data4 = 0x01;
		dds_multiplier = 6;
	}
	else
	{
		data4 = 0x00;
		dds_multiplier = 1;
	}
    uint32_t tmp_freq = freq;
	line1[2] = tmp_freq / 10000000 + 48;
	tmp_freq = tmp_freq % 10000000;
	line1[3] = tmp_freq / 1000000 + 48;
	tmp_freq = tmp_freq % 1000000;
	line1[5] = tmp_freq / 100000 + 48;
	tmp_freq = tmp_freq % 100000;
	line1[6] = tmp_freq / 10000 + 48;
	tmp_freq = tmp_freq % 10000;
	line1[7] = tmp_freq / 1000 + 48;
	tmp_freq = tmp_freq % 1000;
	line1[9] = tmp_freq / 100 + 48;
	tmp_freq = tmp_freq % 100;
	line1[10] = tmp_freq / 10 + 48;
	tmp_freq = tmp_freq % 10;
	line1[11] = tmp_freq + 48;
	if(position & 0x10)
	{
		line1[15] = '<';
		line1[0] = '>';
		
	}
	else
	{
		line1[15] = ' ';
		line1[0] = ' ';
	}
	if((position & 0x0F) <= 2)
	{
		curpos = (position & 0x0F)+1;
	}
	else if((position & 0x0F) >= 6)
	{
		curpos = (position & 0x0F)+3;
	}
	else
	{
		curpos = (position & 0x0F)+2;
	}
	dds_update();	
}

void dds_update(void)
{
	uint32_t dds_freq_word;
	dds_freq_word = (uint32_t)((0x100000000/(float)(DDS_CLK*dds_multiplier))*freq);
	data0 = (uint8_t)(dds_freq_word & 0x000000FF);
	data1 = (uint8_t)((dds_freq_word & 0x0000FF00)>>8);
	data2 = (uint8_t)((dds_freq_word & 0x00FF0000)>>16);
	data3 = (uint8_t)((dds_freq_word & 0xFF000000)>>24);
	dds_load();
}

void decr(void)
{
	switch(position)
	{
	case 0x11:
		freq=freq-10000000;
		break;
	case 0x12:
		freq=freq-1000000;
		break;
	case 0x13:
		freq=freq-100000;
		break;
	case 0x14:
		freq=freq-10000;
		break;
	case 0x15:
		freq=freq-1000;
		break;
	case 0x16:
		freq=freq-100;
		break;
	case 0x17:
		freq=freq-10;
		break;
	case 0x18:
		freq=freq-1;
		break;
	default:
		if(position>1) position--;
		break;
	}
	if(freq>50000000) freq=0;
	freq_update();
	lcd_update();
}

void incr(void)
{
	switch(position)
	{
	case 0x11:
		freq=freq+10000000;
		break;
	case 0x12:
		freq=freq+1000000;
		break;
	case 0x13:
		freq=freq+100000;
		break;
	case 0x14:
		freq=freq+10000;
		break;
	case 0x15:
		freq=freq+1000;
		break;
	case 0x16:
		freq=freq+100;
		break;
	case 0x17:
		freq=freq+10;
		break;
	case 0x18:
		freq=freq+1;
		break;
	default:
		if(position<8) position++;
		break;
	}
	if(freq>50000000) freq=50000000;
	freq_update();
	lcd_update();
}

SIGNAL(SIG_OVERFLOW0) 			// co 2ms
{
	TCNT0 = 193;//193
	if(!((PIN(SWIMP_PORT))&(1<<SWIMP))){
		if(!(sw & 0x02)){
			if(sw & 0x01){
				if(position & 0x10)
				{
					position &= ~0x10; 
				}
				else
				{
					position |= 0x10;
				}
				freq_update();
				lcd_update();
				sw |= 0x02;
			}
			sw |= 0x01;
		}
	}
	else{
		sw &= 0xFC;
	}
	
	if(!((PIN(SWGAIN_PORT))&(1<<SWGAIN))){
		if(!(sw & 0x08)){
			if(sw & 0x04){
				if(PIN(GAIN_PORT)&(1<<GAIN))
				{
					PORT(GAIN_PORT) &= ~(1<<GAIN);
					setting &= 0xFD;
				}
				else
				{
					PORT(GAIN_PORT) |= 1<<GAIN;
					setting |= 0x02;
				}
				ampl_update(adc_avg);
				lcd_update();
				sw |= 0x08;
			}
			sw |= 0x04;
		}
	}
	else{
		sw &= 0xF3;
	}
	
	if(!((PIN(SWSQR_PORT))&(1<<SWSQR))){
		if(!(sw & 0x20)){
			if(sw & 0x10){
				if(PIN(SQR_PORT)&(1<<SQR))
				{
					PORT(SQR_PORT) &= ~(1<<SQR);
					setting &= 0xFE;
				}
				else
				{
					PORT(SQR_PORT) |= 1<<SQR;
					setting |= 0x01;
				}
				sqr_update();
				lcd_update();
				sw |= 0x20;
			}
			sw |= 0x10;
		}
	}
	else{
		sw &= 0xCF;
	}

	if((PIN(IMP_PORT))&(1<<IMPB))
	{
		if(imp_stat != 0x22)
		{
			imp_stat |= 0x10;
			if(imp_stat & 0x01)
			{
				
				imp_stat = 0x22;
				decr();
			}
		}
	}
	else 
	{
		imp_stat &= 0xCC;
	}
	if((PIN(IMP_PORT))&(1<<IMPA))
	{
		if(imp_stat != 0x22)
		{
			imp_stat |= 0x01;
			if(imp_stat & 0x10)
			{
				
				imp_stat = 0x22;
				incr();
			}
		}
	}
	else
	{
		imp_stat &= 0xCC;
	}
	
	adc_it++;
	if((adc_it & 0x0F) == 0x0A)
	{
		if((ADCSRA & (1<<ADSC)) == 0)
		{
			adc_it += 0x10;
			adc_sum += ((uint16_t)ADCL | ((uint16_t)ADCH)<<8);
			if((adc_it & 0xF0) == 0xA0)
			{
				if(adc_avg != (uint16_t)((adc_sum/10)*0.434))
				{
					adc_avg = (uint16_t)((adc_sum/10)*0.434);
					ampl_update(adc_avg);
					lcd_update();
				}
				adc_sum = 0;
				adc_it &= 0x0F;
			}
			ADCSRA |= 1<<ADSC;
		}
		adc_it &= 0xF0;
	}

}
 